/**
 ** scoutCntrlTaskCodels.c
 **
 ** Codels used by the control task scoutCntrlTask
 **
 ** Auteur: 
 ** Date: 
 **
 **/

#include <math.h>
#include <stdio.h>
#include <portLib.h>
#include <h2mathLib.h>
#include "server/scoutHeader.h"
#include "scoutConst.h"

static BOOL getAlarm (int *bilan);

/*------------------------------------------------------------------------
 * scoutGetPosCntrl  -  control codel of CONTROL request GetPos
 *
 * Description:    
 * Report (bilan): OK 
 *
 * Return:    OK ou ERROR
 */

STATUS scoutGetPosCntrl (int *bilan)
{
  SDI_F->setPos.x = SDI_F->odo.robot.xC;
  SDI_F->setPos.y = SDI_F->odo.robot.yC;
  SDI_F->setPos.theta = SDI_F->odo.robot.theta;

  return OK;
}


/*------------------------------------------------------------------------
 * scoutSetPosDev  -  control codel of CONTROL request SetPosDev
 *
 * Description:    
 * Report (bilan): OK 
 *
 * Return:    OK ou ERROR
 */

STATUS scoutSetPosDev (SCOUT_ODO_VAR_STR *setVarTab, int *bilan)
{
  SCOUT_ODO_STR *odoId;
  
  odoId = &SDI_F->odo;
  
  /* initialise la matrice de variance */
  memcpy(SDI_F->odo.odoError.var, setVarTab->var, sizeof(SCOUT_ODO_VAR_STR));
  
  return OK;
}


/*------------------------------------------------------------------------
 * scoutControlDist  -  control codel of CONTROL request SetCmdDist
 *
 * Description:    
 * Report (bilan): OK 
 *
 * Return:    OK ou ERROR
 */

STATUS scoutControlDist (double *dist, int *bilan)
{
  int sens;
    
  /* Determiner la consigne de sens de mouvement */
  sens = SDI_F->ref.backFlag == SCOUT_BACKWARD_MOTION ? -1 : 1;

  /* Re-actualiser la consigne */
  SDI_F->ref.x = SDI_F->ref.x -
			   sens * (*dist - SDI_F->cmd.distPoint) 
			   * sin(SDI_F->odo.robot.theta);
  SDI_F->ref.y = SDI_F->ref.y +
			   sens * (*dist - SDI_F->cmd.distPoint) 
			   * cos(SDI_F->odo.robot.theta);
  
  return OK;
}


/*------------------------------------------------------------------------
 * scoutVerifyConnection  -  
 *
 * Description:    
 * Report (bilan): OK 
 *                 S_scout_UNINITIALIZED
 *                 S_scout_CONNECTION_FAILED
 *                 S_scout_ROBOT_NOT_FOUND
 *
 * Return:    OK ou ERROR
 */

STATUS scoutVerifyConnection (int *bilan)
{
  if (SDI_F->connectStatus == SCOUT_NOT_CONNECTED) {
    *bilan = S_scout_UNINITIALIZED;
    printf ("scout: not connected to Nrobot\n");
    return ERROR;
  }
  return OK;
}



/*------------------------------------------------------------------------
 * scoutTrackCntrl  -  control codel of EXEC request TrackStart
 *
 * Description:    
 * Report (bilan): OK 
 *                 S_scout_POSTER_NOT_FOUND
 *                 S_scout_BAD_REF
 *                 S_scout_CMD_STOP_TRACK
 *                 S_scout_POSTER_NOT_UPDATED
 *
 * Return:    OK ou ERROR
 */

STATUS scoutTrackCntrl (SCOUT_POSTER_NAME *posterName, int *bilan)
{

  /* Il faut etre connecte a NRobot */
  if (scoutVerifyConnection(bilan) == ERROR){
    return ERROR;
  }

  /* Verifier qu'il n'y a pas d'alarme */
  if (getAlarm(bilan))
    return ERROR;

  /* Debloquer le tracking */
  SDI_F->trackStr.scoutCmdTrackMode = SCOUT_TRACKING;
  SDI_F->motionType = SCOUT_SERVO_POS;

  return OK;
}

/*------------------------------------------------------------------------
 * scoutTrackSpeedCntrl  -  control codel of EXEC request TrackSpeedStart
 *
 * Description:    
 * Report (bilan): OK 
 *                 S_scout_POSTER_NOT_FOUND
 *                 S_scout_BAD_REF
 *                 S_scout_CMD_STOP_TRACK
 *                 S_scout_POSTER_NOT_UPDATED
 *
 * Return:    OK ou ERROR
 */

STATUS scoutTrackSpeedCntrl (SCOUT_POSTER_NAME *posterName, int *bilan)
{

  /* Il faut etre connecte a NRobot */
  if (scoutVerifyConnection(bilan) == ERROR){
    return ERROR;
  }

  /* Verifier qu'il n'y a pas d'alarme */
  if (getAlarm(bilan))
    return ERROR;

  /* Debloquer le tracking */
  SDI_F->trackStr.scoutCmdTrackMode = SCOUT_TRACKING;
  SDI_F->motionType = SCOUT_SERVO_SPEED;

  return OK;
}


/*------------------------------------------------------------------------
 * scoutTrackConfigCntrl  -  control codel of EXEC request TrackConfigStart
 *
 * Description:    
 * Report (bilan): OK 
 *                 S_scout_POSTER_NOT_FOUND
 *                 S_scout_BAD_REF
 *                 S_scout_CMD_STOP_TRACK
 *
 * Return:    OK ou ERROR
 */

STATUS scoutTrackConfigCntrl (SCOUT_POSTER_NAME *posterName, int *bilan)
{

  /* Il faut etre connecte a NRobot */
  if (scoutVerifyConnection(bilan) == ERROR){
    return ERROR;
  }

  /* Verifier qu'il n'y a pas d'alarme */
  if (getAlarm(bilan))
    return ERROR;

  /* Debloquer le tracking */
  SDI_F->trackStr.scoutCmdTrackMode = SCOUT_TRACKING;
  SDI_F->motionType = SCOUT_SERVO_CONFIG;

  return OK;
}


/*------------------------------------------------------------------------
 * scoutSetCmdConfigCntrl  -  control codel of CONTROL request SetCmdConfig
 *
 * Description:    
 * Report (bilan): OK 
 *                 S_scout_BAD_CMD_DATA
 *
 * Return:    OK ou ERROR
 */

STATUS scoutSetCmdConfigCntrl (SCOUT_CMD_PARAM_STR *cmd, int *bilan)
{
  int sens;

  /* Verifier qu'on est pas en tracking */
  if (SDI_F->trackStr.scoutCmdTrackMode) {
    *bilan = S_scout_TRACKING_ON;
    return(ERROR);
  }

  /* Verifier la validite des arguments */
  if (cmd->KpLongit < 0 || cmd->KiLongit < 0 ||
      cmd->KpTransv < 0 || cmd->KiTransv < 0 || 
      cmd->longitErrorMax < 0 || cmd->transvErrorMax < 0 || 
      cmd->longitMaxErrSum < 0 || cmd->transvMaxErrSum < 0) {
    *bilan = S_scout_BAD_CMD_DATA;
    return (ERROR);
  }

  /* Determiner la consigne de sens de mouvement */
  sens = SDI_F->ref.backFlag == SCOUT_BACKWARD_MOTION ? -1 : 1;

  /* Re-actualiser la consigne */
  SDI_F->ref.x = SDI_F->ref.x + 
		 sens * (cmd->distPoint - SDI_F->cmd.distPoint)
		 * cos(SDI_F->odo.robot.theta);
  SDI_F->ref.y = SDI_F->ref.y + 
		 sens * (cmd->distPoint - SDI_F->cmd.distPoint) 
		 * sin(SDI_F->odo.robot.theta);
  
  return OK;
}


/*========================= STATIC FUNCTIONS ========================= */

static BOOL getAlarm (int *bilan)
{
  if (SDI_F->alarms.nbAlarms == 0)
    return FALSE;

  if (SDI_F->alarms.chocFlag)
    *bilan = S_scout_CHOC_BUMPER;
  else if (SDI_F->alarms.lowBatteryFlag)
    *bilan = S_scout_LOW_BATTERY;
  else if (SDI_F->alarms.joystickInUseFlag)
    *bilan = S_scout_JOYSTICK_IN_USE;
  else if (SDI_F->alarms.estopDownFlag)
    *bilan = S_scout_ESTOP_DOWN;
  else if (SDI_F->alarms.motionErrorFlag)
    *bilan = S_scout_MOTION_ERROR;
  else if (SDI_F->alarms.noMotionFlag)
    *bilan = S_scout_NO_MOTION;
  else if (SDI_F->alarms.importantDriftFlag)
    *bilan = S_scout_IMPORTANT_DRIFT;
  else if (SDI_F->alarms.refNotUpdatedFlag)
    *bilan = S_scout_REF_NOT_UPDATED;
  else
    *bilan = S_scout_UNKNOWN_ALARM;
  return TRUE;
}




/*------------------------------------------------------------------------
 * scoutSetPosCntrl  -  control codel of EXEC request SetPos
 *
 * Description:    
 * Report (bilan): OK 
 *                 S_scout_TRACKING_ON
 *
 * Return:    OK ou ERROR
 */

STATUS scoutSetPosCntrl (SCOUT_SET_POS_STR *setPos, int *bilan)
{
  if (scoutVerifyConnection(bilan) == ERROR){
    return ERROR;
  }
  return OK;
}


/*------------------------------------------------------------------------
 * scoutGoToCntrl  -  control codel of CONTROL request GoTo
 *
 * Description:    
 * Report (bilan): OK 
 *                 S_scout_BAD_REF_DATA
 *                 S_scout_TRACKING_ON
 *
 * Return:    OK ou ERROR
 */

STATUS scoutGoToCntrl (SCOUT_REF_STR *pRefStr, int *bilan)
{

  /* Verifier qu'on est pas en tracking */
  if (SDI_F->trackStr.scoutCmdTrackMode) {
    *bilan = S_scout_TRACKING_ON;
    return(ERROR);
  }
    
  /* Verifier qu'il n'y a pas d'alarme */
  if (getAlarm(bilan))
    return ERROR;

  /* Verifier la validite des arguments */
  if (pRefStr->vmax <= 0. || pRefStr->vmax > SCOUT_VMAX_ROBOT ||
      pRefStr->wmax <= 0. || pRefStr->wmax > SCOUT_WMAX_ROBOT ||
      pRefStr->linAccelMax <= 0. || pRefStr->linAccelMax > SCOUT_AMAX_LIN_ROBOT ||
      pRefStr->angAccelMax <= 0. || pRefStr->angAccelMax > SCOUT_AMAX_ANG_ROBOT ||
      pRefStr->v != 0. || pRefStr->w != 0. ) {
    *bilan = S_scout_BAD_REF_DATA;
    return (ERROR);
  }

  /* Limiter l'angle de consigne */
  pRefStr->theta =    (double) angleLimit ((double) pRefStr->theta);

  SDI_F->motionType = SCOUT_SERVO_POS;
  SDI_F->trackStr.scoutCmdTrackMode = SCOUT_NO_TRACKING;

  return OK;
}


/*------------------------------------------------------------------------
 * scoutGoToSpeedCntrl  -  control codel of CONTROL request GoToSpeed
 *
 * Description:    
 * Report (bilan): OK 
 *                 S_scout_BAD_REF_DATA
 *                 S_scout_TRACKING_ON
 *
 * Return:    OK ou ERROR
 */

STATUS scoutGoToSpeedCntrl (SCOUT_SPEED_REF_STR *speedRef, int *bilan)
{
  /* Verifier qu'on est pas en tracking */
  if (SDI_F->trackStr.scoutCmdTrackMode) {
    *bilan = S_scout_TRACKING_ON;
    return(ERROR);
  }
    
  /* Verifier qu'il n'y a pas d'alarme */
  if (getAlarm(bilan))
    return ERROR;

  /* XXXXXXX NO STOP !! */
  SDI_F->trackStr.scoutCmdTrackMode = SCOUT_TRACKING;
  SDI_F->motionType = SCOUT_SERVO_SPEED;

  return OK;
}

/*------------------------------------------------------------------------
 * scoutSonarsFiringOrderCntrl  -  control codel of CONTROL request SetSonarsFiringOrder
 *
 * Description:    
 * Report (bilan): OK 
 *
 * Return:    OK ou ERROR
 */

STATUS scoutSonarsFiringOrderCntrl (SCOUT_SONARS_FIRING_ORDER *sonarsFiring,
				    int *bilan)
{
  int i;

  for(i=0; i<SCOUT_NB_SONARS; i++) {
    if (sonarsFiring->firedSonars[i] < 0 ||
	(sonarsFiring->firedSonars[i] >= SCOUT_NB_SONARS &&
	 sonarsFiring->firedSonars[i] != 255)) {
      *bilan = S_scout_BAD_CMD_DATA;
      return ERROR;
    }
  }
  return OK;
}
