/**
 ** scoutPumpTaskCodels.c
 **
 ** Codels appeles par la tache scoutPumpTask
 **
 ** Auteur: 
 ** Date: 
 **
 **/

#include <stdio.h>
#include <string.h>

#include <portLib.h>
#include <h2mathLib.h>

#include "server/scoutHeader.h"
#include "scoutConst.h"

static  SCOUT_REF_STR scoutOrders;
static  SCOUT_SPEED_REF_STR scoutSpeedOrders;

static BOOL 
findTrackedPoster (SCOUT_POSTER_NAME *posterName, int *bilan);
static ACTIVITY_EVENT 
scoutSmoothStopTracking (SCOUT_POSTER_NAME *posterName, int *bilan);

#ifndef max
#define max(x,y) ((x)<(y)?(y):(x))
#endif
/*----------------------------------------------------------------------
 *
 *                          START TRACKINGS
 *
 ----------------------------------------------------------------------*/

/*
 *  TRACKING POSITION
 */
ACTIVITY_EVENT scoutPumpTrackStart (SCOUT_POSTER_NAME *posterName, int *bilan)
{
  /* Recherche du poster */
  if (!findTrackedPoster(posterName, bilan))
    return ETHER; 

  /* Premiere lecture */
  if (posterRead ((POSTER_ID)scoutDataStrId->trackStr.posterId, 
		  0, (void *) &scoutOrders,
		  sizeof (SCOUT_REF_STR)) != sizeof (SCOUT_REF_STR)) {
    *bilan = S_scout_POSTER_NOT_FOUND;
    return ETHER; 
  }
  scoutDataStrId->ref.numRef = 1;

  return EXEC;
}

/*
 *  TRACKING CONFIGURATION
 */
ACTIVITY_EVENT scoutPumpTrackConfigStart (SCOUT_POSTER_NAME *posterName, int *bilan)
{
  ACTIVITY_EVENT event;

  event = scoutPumpTrackStart(posterName, bilan);

  return event;
}

/*
 *  TRACKING SPEED
 */
ACTIVITY_EVENT scoutPumpTrackSpeedStart (SCOUT_POSTER_NAME *posterName, int *bilan)
{
  /* Recherche du poster */
  if (!findTrackedPoster(posterName, bilan))
    return ETHER; 
  
  /* Premiere lecture */
  if (posterRead ((POSTER_ID)scoutDataStrId->trackStr.posterId, 
		  0, (void *) &scoutSpeedOrders,
		  sizeof (SCOUT_SPEED_REF_STR)) != sizeof (SCOUT_SPEED_REF_STR)) {
    *bilan = S_scout_POSTER_NOT_FOUND;
    return ETHER; 
  }
  scoutDataStrId->speedRef.numRef = 1;

  return EXEC;
}


/*----------------------------------------------------------------------
 *
 *                          EXEC  TRACKINGS
 *
 ----------------------------------------------------------------------*/

/*
 *  TRACKING POSITION (x,y)
 */

ACTIVITY_EVENT scoutPumpTrackExec (SCOUT_POSTER_NAME *posterName, int *bilan)
{
  static int noChange = 0;
  static int prevNumRef = 0;
  int prevScoutNumRef;

  /* Interruption interne */
  if (scoutDataStrId->trackStr.scoutCmdTrackMode == SCOUT_NO_TRACKING) {
    *bilan = EXEC_TASK_BILAN(SCOUT_EXECTASK_NUM);
    scoutDataStrId->ref.numRef = 0; /* alarm detectee */
    return END;
  }

  /* Lire les consignes sur le poster externe */
  if (posterRead ((POSTER_ID)scoutDataStrId->trackStr.posterId, 0, 
		  (void *) &scoutOrders,
		  sizeof (SCOUT_REF_STR)) != sizeof (SCOUT_REF_STR)) {
    *bilan = errnoGet();
    return END;
  }
    
  /* Si les vitesses de consignes sont non-nulles, 
     verifier que la consigne est mise a jour */
  if (scoutOrders.numRef == prevNumRef) {
    if (fabs(scoutOrders.v) > 0.001 || fabs(scoutOrders.w) > 0.001) {
      noChange++;
      /*      printf ("noChange %d (%d %g %g)\n", noChange, scoutOrders.numRef,
	      scoutOrders.v, scoutOrders.w);*/
      if (noChange > SDI_F->scoutWaitRefMax) {
	*bilan = S_scout_POSTER_NOT_UPDATED;
	printf ("scoutPumpTask : no ref change for %d times !\n", noChange);
	noChange = 0;
	return END;
      }
    }
    else 
      noChange = 0;
  }

  /* La consigne a changee */
  else {
    prevNumRef = scoutOrders.numRef;
    noChange = 0;

    /* Verifier la validite des arguments */
    if (scoutOrders.vmax <= 0. || 
	scoutOrders.vmax > SCOUT_VMAX_ROBOT ||
	scoutOrders.wmax <= 0. || 
	scoutOrders.wmax > SCOUT_WMAX_ROBOT ||
	scoutOrders.linAccelMax <= 0. ||
	scoutOrders.linAccelMax > SCOUT_AMAX_LIN_ROBOT ||
	scoutOrders.angAccelMax <= 0. ||
	scoutOrders.angAccelMax > SCOUT_AMAX_ANG_ROBOT) {
      *bilan = S_scout_BAD_REF;
      return (END);
    }
  }

  /* Transmettre la consigne */
  prevScoutNumRef = scoutDataStrId->ref.numRef;
  scoutDataStrId->ref = scoutOrders;

  /* Signale la mise a jour de la consigne */
  scoutDataStrId->ref.numRef = prevScoutNumRef == 0 ? 1 : 0;
  return(EXEC);
} 


/*
 *  TRACKING SPEED (v,w)
 */

ACTIVITY_EVENT scoutPumpTrackSpeedExec (SCOUT_POSTER_NAME *posterName, int *bilan)
{
  static int noChange = 0;
  static int noChangeReal = 0;
  static int prevNumRef = 0;
  int prevScoutNumRef;
  double dv, dw;
/*   int nbPeriodRemain; */
  static double vactual=0,wactual=0,incv=0,incw=0;
  static double prevVRef=0, prevWRef=0;


  /* Interruption interne */
  if (scoutDataStrId->trackStr.scoutCmdTrackMode == SCOUT_NO_TRACKING) {
    *bilan = EXEC_TASK_BILAN(SCOUT_EXECTASK_NUM);
    scoutDataStrId->speedRef.numRef = 0; /* alarme detectee */
    return(END);
  }

  /* Lire les consignes sur le poster externe */
  if (posterRead ((POSTER_ID)scoutDataStrId->trackStr.posterId, 0, 
		  (void *) &scoutSpeedOrders,
		  sizeof (SCOUT_SPEED_REF_STR)) != sizeof (SCOUT_SPEED_REF_STR)) {
    *bilan = errnoGet();
    return(END);
  }
    
  /* Si les vitesses de consignes sont non-nulles, 
     verifier que la consigne est mise a jour */
  if (scoutSpeedOrders.numRef == prevNumRef) {
    noChangeReal++;
    prevVRef = scoutSpeedOrders.v;
    prevWRef = scoutSpeedOrders.w;

    if (fabs(scoutSpeedOrders.v) > 0.001 || fabs(scoutSpeedOrders.w) > 0.001) {
      noChange++;
      /* printf ("noChange %d (%d %g %g)\n", noChange, scoutSpeedOrders.numRef,
	 scoutSpeedOrders.v, scoutSpeedOrders.w);*/
      if (noChange > SDI_F->scoutWaitRefMax) {
	*bilan = S_scout_POSTER_NOT_UPDATED;
	printf ("scoutPumpTask : no ref change for %d times !\n", noChange);
	noChange = 0;
	return END;
      }
    }
    else {
      noChange = 0;
    }
  }

  /* La consigne a changee */
  else {
    prevNumRef = scoutSpeedOrders.numRef;
    noChange = 0;
    noChangeReal = 0;
/*     vactual = SDI_F->odo.robot.v ; */
/*     wactual = SDI_F->odo.robot.w; */
    vactual = prevVRef;
    wactual = prevWRef;
    if (scoutSpeedOrders.updatePeriod <= 1)
      scoutSpeedOrders.updatePeriod = 1;
    incv = (scoutSpeedOrders.v - vactual)/scoutSpeedOrders.updatePeriod;
    incw = (scoutSpeedOrders.w - wactual)/scoutSpeedOrders.updatePeriod;
  }

  /* Calcul une consigne intermediaire */
  if (noChangeReal < scoutSpeedOrders.updatePeriod) {
    dv = incv * (noChangeReal+1);
    dw = incw * (noChangeReal+1);
    scoutSpeedOrders.v = vactual + dv;
    scoutSpeedOrders.w = wactual + dw;
  }

/*   dv = scoutSpeedOrders.v - SDI_F->odo.robot.v; */
/*   dw = scoutSpeedOrders.w - SDI_F->odo.robot.w; */
/*   nbPeriodRemain = scoutSpeedOrders.updatePeriod - noChangeReal; */
/*   if (nbPeriodRemain <= 0) { */
/*     nbPeriodRemain = 0; */
/*     dv = 0.0; */
/*     dw = 0.0; */
/*   } */
/*   else { */
/*     dv = dv/nbPeriodRemain; */
/*     dw = dw/nbPeriodRemain; */
/*   } */
/*   scoutSpeedOrders.v = scoutSpeedOrders.v -  (nbPeriodRemain-1) * dv; */
/*   scoutSpeedOrders.w = scoutSpeedOrders.w -  (nbPeriodRemain-1) * dw; */

  /* Transmettre la consigne */
  prevScoutNumRef = scoutDataStrId->speedRef.numRef;
  memcpy ((void *) &scoutDataStrId->speedRef, (void *) &scoutSpeedOrders, 
	  sizeof (SCOUT_SPEED_REF_STR));

  /* Signale la mise a jour de la consigne */
  scoutDataStrId->speedRef.numRef = prevScoutNumRef == 0 ? 1 : 0;
  return(EXEC);
} 

/*
 *  TRACKING CONFIGURATION (x,y,theta)
 */

ACTIVITY_EVENT scoutPumpTrackConfigExec (SCOUT_POSTER_NAME *posterName, int *bilan)
{
  return scoutPumpTrackExec(posterName, bilan);
} 


/*----------------------------------------------------------------------
 *
 *                          END TRACKINGS
 *
 ----------------------------------------------------------------------*/

/*
 *  TRACKING POSITION
 */

ACTIVITY_EVENT scoutPumpTrackStop (SCOUT_POSTER_NAME *posterName, int *bilan)
{
  return scoutSmoothStopTracking(posterName, bilan);
}

/*
 *  TRACKING SPEED
 */
ACTIVITY_EVENT scoutPumpTrackSpeedStop (SCOUT_POSTER_NAME *posterName, int *bilan)
{
   return scoutSmoothStopTracking(posterName, bilan);
}

/*
 *  TRACKING CONFIG
 */
ACTIVITY_EVENT scoutPumpTrackConfigStop (SCOUT_POSTER_NAME *posterName, int *bilan)
{
   return scoutSmoothStopTracking(posterName, bilan);
}


/* =====================  LOCALES FUNCTIONS   =========================== */

/**
 **  scoutStopTracking  -  Computes a reference to smooth stop the tracking
 **                       and gets back to NO_TRACKING mode
 **
 **
 **/

static ACTIVITY_EVENT 
scoutSmoothStopTracking (SCOUT_POSTER_NAME *posterName, int *bilan)
{
  SCOUT_REF_STR *refId;                /* Id structure de consignes */
  SCOUT_POS_STR *robotId;        /* Id parametres cinematiques principaux */
  SCOUT_CMD_PARAM_STR *cmdId;          /* Id param cmdmetriques du robot */
  double dtArret;                /* Temps d'arret */
  double dtRetard;               /* Temps de prise en compte de la consigne */
  double dsArret;                /* Longueur d'arret */
  double dtScout;                 /* Periode d'asserv de la scout */
  double angle;                  /* Moyenne entre angles d'init et d'arret */
  double xC, yC;
  int sens;

  /* Obtenir les pointeurs vers les structures de donnees de la scoutmotion */
  robotId = &(scoutDataStrId->odo.robot);
  refId = &scoutDataStrId->ref;
  cmdId = &scoutDataStrId->cmd;

  /* Determination de la periode d'asserv de la scout */
  dtScout = EXEC_TASK_PERIOD(SCOUT_EXECTASK_NUM);
  
  /* Temps d'arret minimal (tps de decel) */
  dtArret = max (fabs(robotId->v) / SCOUT_AMAX_LIN_ROBOT,
		 fabs(robotId->w) / SCOUT_AMAX_ANG_ROBOT);
  
  /* Temps de prise en compte (v = cst) */
  dtRetard =  1 * dtScout;
  
  /* Angle final (avec periode de prise en compte) */
  refId->theta = robotId->theta + robotId->w * (dtArret / 2 + dtRetard);
  
  /* Angle moyen */
  angle = (robotId->theta + refId->theta) / 2;
  
  /* Longueur d'arret */
  dsArret = robotId->v * (dtArret / 2 + dtRetard);
  
  /* Prevision position du centre */
  xC = robotId->xC + dsArret * cos (angle);
  yC = robotId->yC + dsArret * sin (angle);
  
  /* Determiner le sens de mouvement */
  if (((robotId->x - robotId->xC) * cos(robotId->theta) 
       + (robotId->y - robotId->yC) * sin(robotId->theta)) < 0.0) {
    sens = -1.0;
  } else {
    sens = 1.0;
  }
    
  /* Consigne */
  refId->x = xC + sens * cmdId->distPoint * cos (refId->theta);
  refId->y = yC + sens * cmdId->distPoint * sin (refId->theta);

  /* Vitesses de consigne nulles */
  refId->v = 0. ;
  refId->w = 0. ;

  /* Vitesses max */
  refId->vmax = SCOUT_VMAX_ROBOT;
  refId->wmax = SCOUT_WMAX_ROBOT;

  /* Accelerations (ie decelerations) max */
  refId->linAccelMax = SCOUT_AMAX_LIN_ROBOT ;
  refId->angAccelMax = SCOUT_AMAX_ANG_ROBOT ;

  /* Indiquer l'arret des tracking */
  scoutDataStrId->speedRef.numRef = 0;
  scoutDataStrId->ref.numRef = 0;
  scoutDataStrId->trackStr.scoutCmdTrackMode = SCOUT_NO_TRACKING;
  return ETHER;

}

/**
 **  findTrackedPoster  - Recherche des posters de consigne 
 **                       
 **
 **
 **/

static BOOL 
findTrackedPoster (SCOUT_POSTER_NAME *posterName, int *bilan)
{
  POSTER_ID posterPumpedId;
  
  /* Identificateur du poster */
  if (posterFind((char *) posterName, &posterPumpedId) != OK)  {
    *bilan = S_scout_POSTER_NOT_FOUND;
    return(FALSE); 
  }
  scoutDataStrId->trackStr.posterId = (int) posterPumpedId;
  return TRUE;
}
