/**
 ** Copyright (c) 1999 LAAS/CNRS 
 **
 ** Matthieu Herrb - Tue Jun 15 1999
 **
 ** $Source$
 ** $Revision$
 ** $Date$
 ** 
 **/

#include <portLib.h>
#include <errnoLib.h>
#include <math.h>
#include <string.h>

#include <h2mathLib.h>

#include "server/scoutHeader.h"
#include "server/scoutPosterWriteLibProto.h"

#include "nomadic/Nclient.h"

#include "scoutConst.h"
#include "scoutStruct.h"

static double cosTheta[SCOUT_NB_SONARS];
static double sinTheta[SCOUT_NB_SONARS];
	
/*----------------------------------------------------------------------*/
STATUS
scoutComputeSonarsCoord(int *bilan)
{
    int i;
    double th;
    double radius = SCOUT_DIAMETER/2.;

    /* Compute sonars position */
    th = 0;
    for (i=0; i<SCOUT_NB_SONARS; i++) {
      SDI_F->sonarsCoord.sonarConfig[i].x = cos(th) * radius;
      SDI_F->sonarsCoord.sonarConfig[i].y = sin(th) * radius;
      SDI_F->sonarsCoord.sonarConfig[i].theta = th;

      /* Pre-computes sin and cos */
      cosTheta[i] = cos(SDI_F->sonarsCoord.sonarConfig[i].theta);
      sinTheta[i] = sin(SDI_F->sonarsCoord.sonarConfig[i].theta);

      th += 2*PI/SCOUT_NB_SONARS;  /* XXXX VERIFIER SENS DE ROTATION
				      ET POSITION 1ER SONARS */
    }

    /* Update poster */
    if (scoutSensorsConfigsonarsCoordPosterWrite(SCOUT_SENSORSCONFIG_POSTER_ID,
						 &SDI_F->sonarsCoord) == ERROR) {
	*bilan = errnoGet();
	return ERROR;
    }

    return OK;
}

/*----------------------------------------------------------------------*/

/* Bumpers position on the robot circonference */
#define SCOUT_NB_BUMPERS_PLACES 8 
#define SCOUT_BUMPERS_ANGLE_POS {0, 1, 3, 4, 5, 7}

STATUS
scoutComputeBumpersCoord(int *bilan)
{
    int i;
    double dth, th;
    double radius = SCOUT_DIAMETER/2.;
    int bumpersPlaces[SCOUT_NB_BUMPERS] = SCOUT_BUMPERS_ANGLE_POS;


    /* Compute bumpers position */
    dth = 2*PI/SCOUT_NB_BUMPERS_PLACES;
    for (i=0; i<SCOUT_NB_BUMPERS; i++) {
      th = bumpersPlaces[i] * dth;

      SDI_F->bumpersPos.bumperPos[i].x = cos(th) * radius;
      SDI_F->bumpersPos.bumperPos[i].y = sin(th) * radius;

    }

    /* Update poster */
    if (scoutSensorsConfigbumpersPosPosterWrite(SCOUT_SENSORSCONFIG_POSTER_ID,
						&SDI_F->bumpersPos) == ERROR) {
	*bilan = errnoGet();
	return ERROR;
    }

    return OK;
}

/*----------------------------------------------------------------------*/
STATUS
scoutComputeSonarsEcho(int *bilan)
{
    int i;
    SCOUT_POS *robotTab; /* SONARS echo (points in robot) */
    SCOUT_POS *globalTab;/* SONARS echo (points in lab) */
    SCOUT_POS_STR *pos;
    double measure, rcos, rsin;

    /* Copie Sonars */
    robotTab = SDI_F->sonarsEchoFrameRobot.point;
    globalTab = SDI_F->sonarsEchoFrameGlobal.point;
    pos = &(SDI_F->odo.robot);

    rcos = h2cos(pos->theta);
    rsin = h2sin(pos->theta);

    for (i = 0; i < SCOUT_NB_SONARS; i++) {

      /* XXXX TESTER LES SONARS ACTIFS */
#if 0 
	sonarsSet = &scoutState->SonarController.SonarSet[i];
	if (sonarsSet->DataActive == FALSE) {
	    continue;
	}
	if (sonarsSet->Sonar[i].Reading == N_SONAR_TIMEOUT) {
	  SDI_F->sonarsEcho.rho[i] = SCOUT_SONARS_OUT_OF_RANGE;
	  continue;
	}
#endif
	if (State[STATE_SONAR_0+i] == SCOUT_SONARS_RHO_MAX) {
	  SDI_F->sonarsEcho.rho[i] = SCOUT_SONARS_OUT_OF_RANGE;
	  continue;
	}

	measure = IN_TO_M(State[STATE_SONAR_0+i]);
	SDI_F->sonarsEcho.rho[i] = measure;
	
	if (measure > 0) {
	  /* Calcul des echos dans le repere local */
	  robotTab[i].x 
	    = SDI_F->sonarsCoord.sonarConfig[i].x + measure*cosTheta[i];
	  robotTab[i].y 
	    = SDI_F->sonarsCoord.sonarConfig[i].y + measure*sinTheta[i];
	  
	  /* Calcul des echos dans le repere global */
	  globalTab[i].x 
	    = pos->xC + robotTab[i].x*rcos - robotTab[i].y*rsin;
	  globalTab[i].y 
	    = pos->yC + robotTab[i].x*rsin + robotTab[i].y*rcos;
	}
	else {
	  /* Calcul des echos dans le repere local */
	  robotTab[i].x  = 0.0;
	  robotTab[i].y  = 0.0;
	  
	  /* Calcul des echos dans le repere global */
	  globalTab[i].x = 0.0;
	  globalTab[i].y = 0.0;
	}
    } /* for */
    return OK;
}

/*----------------------------------------------------------------------*/
STATUS
scoutComputeBumpersEcho(int *bilan)
{
  int i;
  for(i=0;i<SCOUT_NB_BUMPERS;i++) {
    if(State[STATE_BUMPER] & (1L << i))
      SDI_F->bumpersState.bumper[i] = SCOUT_TRUE;
    else
      SDI_F->bumpersState.bumper[i] = SCOUT_FALSE;
  }
  return OK;
}
