#ifndef SCOUT_CONST_H
#define SCOUT_CONST_H

#define DECI_IN_TO_M(in) (0.00254*(in))
#define IN_TO_M(in)      (0.0254*(in))
#define M_TO_DECI_IN(m)  ((m)/0.00254)
#define DECI_DEG_TO_RAD(ddeg) (3.1415*(ddeg)/1800)
#define RAD_TO_DECI_DEG(rd)   (1800*(rd)/3.1415)


#define SCOUT_DIAMETER DECI_IN_TO_M(136)     /* 0.34544 */

#ifdef CONNECT_CLIENT
#define SCOUT_DEFAULT_CONNECT_MODE  SCOUT_NCLIENTSIMUL_CONNECTION
#define SCOUT_DEFAULT_CONNECT_HOST "rantanplan"
#else
#define SCOUT_DEFAULT_CONNECT_MODE  SCOUT_NDIRECT_CONNECTION
#define SCOUT_DEFAULT_CONNECT_HOST "/dev/ttyS0"
#endif

#define SCOUT_DEFAULT_CONNECT_PORT 7019
#define SCOUT_DEFAULT_CONNECT_ROBOT_ID  1

#define SCOUT_DEFAULT_LINEAR_SPEED  SCOUT_VMAX/3
#define SCOUT_DEFAULT_ANGULAR_SPEED  SCOUT_WMAX/6

#ifndef STATE_VEL_RIGHT
#define STATE_VEL_RIGHT STATE_VEL_TRANS
#define STATE_VEL_LEFT STATE_VEL_STEER
#endif

#define SCOUT_POSTER_REF_NO_UPDATE_MAX 40  /* 40 * 4 tic */

/*---------------- PARAMETRES GEOMETRIQUES PAR DEFAUT ----------------------*/

/* Offset angulaire entre le repere a` la mode Nomadic (Y vers l'avant) 
 * et le repere a` la mode LAAS (X vers l'avant) */
#define SCOUT_THETA_OFFSET (M_PI/2.0)

/* Distance du point controle a l'axe des roues */
#define SCOUT_DIST_TO_CONTROLED_POINT      0.2

/* Distance min pour un point controle en avant de l'axe des roues */
#define SCOUT_DIST_CONTROLE_MIN            0.05

/* Entre-axe virtuel */
#define SCOUT_MOT_AXIS_WIDTH 0.5

/*---------- PARAMETRES PAR DEFAUT DE LA BOUCLE D'ASSERVISSEMENT ------------*/

/* Les 4 parametres ci-dessous sont donne's par 
   <http://www.robots.com/scout.htm> */

/* Vitesse lineaire max du robot */
#define  SCOUT_VMAX_ROBOT     DECI_IN_TO_M(400)    /* 1m/s */

/* Vitesse angulaire max du robot */
#define  SCOUT_WMAX_ROBOT     2*SCOUT_VMAX_ROBOT/SCOUT_DIAMETER  /* 5.78rd/s */

/* Acceleration max lineaire du robot */
#define  SCOUT_AMAX_LIN_ROBOT               5. /* m/s^2 */

/* Acceleration max angulaire du robot */
#define  SCOUT_AMAX_ANG_ROBOT               2. /* rad/s^2 */

/* Correction vitesse lineaire, gain proportionnel erreur longitudinale */
#define  SCOUT_KP_LONGIT                    0.2

/* Correction vitesse lineaire, gain integral erreur longitudinale */
#define  SCOUT_KI_LONGIT                    0.0

/* Correction vitesse angulaire, gain proportionnel, erreur transversale */
#define  SCOUT_KP_TRANSV                    0.2

/* Correction vitesse angulaire, gain integral, erreur transversale */
#define  SCOUT_KI_TRANSV                    0.0

/* Correction vitesse angulaire, gain proportionnel erreur angulaire */
#define  SCOUT_KP_ANG                      0.6

/* Correction vitesse angulaire, gain integral erreur angulaire */
#define  SCOUT_KI_ANG                       0

/* Max erreur longitud, pour prise en compte composante integrale vit. lin. */
#define  SCOUT_LONGIT_ERROR_MAX             0.1

/* Max. valeur de l'integrale d'erreurs longitudinales */
#define  SCOUT_LONGIT_MAX_ERR_SUM           0.5

/* Max erreur transverse, prise compte integrale */
#define  SCOUT_TRANSV_ERROR_MAX             0.1

/* Max. valeur de l'integrale d'erreurs transverses */
#define  SCOUT_TRANSV_MAX_ERR_SUM           0.5

/* Max erreur angulaire, pour prise en compte composante integrale vit. lin. */
#define  SCOUT_ANG_ERROR_MAX             0.15

/* Max. valeur de l'integrale d'erreurs angulaires */
#define  SCOUT_ANG_MAX_ERR_SUM           0.3

/* Variances sur la position a l'init */
#define SCOUT_VAR_COEFF                     0.005
#define SCOUT_INIT_VAR_X                    0.00
#define SCOUT_INIT_VAR_Y                    0.00
#define SCOUT_INIT_VAR_THETA                0.000

/*--------ECARTS AU DELA DESQUELS LE TRACKING EST INTERROMPU----  */

#define SCOUT_LONGIT_GAP_MAX           2
#define SCOUT_TRANSV_GAP_MAX           2
#define SCOUT_ANG_GAP_MAX              3

/* -------------------- SONARS FIRING ORDER ------------------------------ */

#define SCOUT_SONARS_RHO_MAX 255  /* selon programme exemple dans doc */

#define SCOUT_SONARS_OUT_OF_RANGE 0.0

#if 0
 /* ordre de tir par defaut de Nomadic Technologies (les premiers et les derniers
    sonars tirants ont de fortes chances de se "leurrer" (l'un entend l'echo de
    l'autre)) */
#define SCOUT_DEFAULT_SONARS_FIRING_ORDER {0, 2, 15, 1, 14, 3, 13, 4, 12, 5,
                                           11, 6, 10, 7, 9, 8}
#endif

/* ordre de tir gardant un ecart angulaire constant entre les sonars tirant
   consecutivement */
#define SCOUT_DEFAULT_SONARS_FIRING_ORDER {0, 7, 14, 5, 12, 3, 10, 1, 8, 15, 6, 13, 4, 11, 2, 9}

#define SCOUT_DEFAULT_SONARS_FIRING_RATE 10

#endif /* SCOUT_CONST_H */
